function   outza = CStest(histInfl,G1_sp,G1_spi, G1_si, impact_sp, impact_spi, impact_si, ...
                    Sig_sp, Sig_spi, Sig_si, ind, aa);
% Input: 
% G1... impact... - output from Gensys for sp, spi and si
% ind = [1,2,3]-3 models; [1,2] - sp,spi; [1,3] - sp,si ; [3,2] - si,spi;
% aa = [5;10;20;30;50;100]; %[5;6;7;8;9;10]
% Output: 
% outza - table with test statistics, critical values, and SC loss measures
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Historical Inflation 
ld_pc_Y(:,1) = histInfl(2:end); % inflation, t
ld_pc_Y(:,2) = histInfl(1:end-1); % inflation, t-1

TT = size(ld_pc_Y,1); % define length of data sample

%  Initial parameters: 
ls=[20]; %ls=[5;8;10;16;20];
n_ls = size(ls,1);

 
n_aa = size(aa,1);
Unum=20;     % density of U grid 
bootnum=100;  % number of boot replications for CV calc 
simtot=1;

% for empirical application 
% grid size and coverage: 
%Umin=min(ld_pc_Y)';
%Umax=max(ld_pc_Y)';
Umin = quantile(ld_pc_Y,.25)';
Umax = quantile(ld_pc_Y,.75)';
Uinc=(Umax-Umin)./Unum;

outzaa=[];

% define length of the simulated samples, given TT, where TT is actual data sample used 
outza = zeros(n_aa*n_ls,n_ls+size(ind,2));

%3. simulate data according to some different models, where SS is the length of the 
         
 % 3.1. simulated sample series
 x_sp = hist_sim(G1_sp,impact_sp,Sig_sp,aa(end)*TT+1,1); %1-pi_t, 2-y_t, 4- a_t
 x_spi = hist_sim(G1_spi,impact_spi,Sig_spi,aa(end)*TT+1,1);
 x_si = hist_sim(G1_si,impact_si,Sig_si,aa(end)*TT+1,1);

 % 3.2. simulated output gap and inflation
 YdatST =[x_sp(2:end,1),x_spi(2:end,1),x_si(2:end,1)]; % inflation, t
 IdatST =[x_sp(1:end-1,1),x_spi(1:end-1,1),x_si(1:end-1,1)]; % inflation, t-1
 YdatST = YdatST(:,ind);
 IdatST = IdatST(:,ind);
 Oboy = zeros(n_aa*n_ls,size(YdatST,2)-1);

         [x_a,den_a]=kden(ld_pc_Y); % actual density
         obs_kd = (1:50:size(YdatST,1))';  
         [x_ys,den_ys]=kden(YdatST(obs_kd,:)); % simulated inflation density
         %plot
%          figure;  plot(x_a(:,1),den_a(:,1),x_ys(:,1),den_ys(:,1),x_ys(:,2),den_ys(:,2),...
%              x_ys(:,3),den_ys(:,3)); legend('hist','sp','spi','si'); 
%          figure; plot(ld_pc_Y(:,1),ld_pc_Y(:,2),'.', YdatST(obs_kd,:), IdatST(obs_kd,:),'.'); 
%          legend('hist','sp','spi','si'); 
for tsi = 1:n_aa;
   SS = aa(tsi)*TT;
   YdatS = YdatST(1:SS,:);
   IdatS = IdatST(1:SS,:);
 
         % 3.3. now, caculate all stats over U range 
         [outttu simF] = zee2o(ld_pc_Y(:,1),ld_pc_Y(:,2),YdatS,IdatS,Umin,Uinc,Unum);
         % average up across the u, rational for averagin described on p.6 
         oboy=mean(outttu)';
         % 3.4. calculate statistics 
         ZZ=ZTstats(oboy);
         Oboy((tsi-1)*n_ls+1:tsi*n_ls,:) = repmat(oboy',n_ls,1); 
         outza((tsi-1)*n_ls+1:tsi*n_ls,[1,6:6+size(IdatST,2)-1])=[ZZ*ones(n_ls,1) repmat(mean(simF),n_ls,1)]; 
end;

%        4.  Now, bootstrap the critical values
ZZZb1=zeros(bootnum,n_aa*n_ls); ZZZb2=zeros(bootnum,n_aa*n_ls);

for booti = 1:bootnum;
    % 4.1. simulated sample series
    x_sp = hist_sim(G1_sp,impact_sp,Sig_sp,aa(end)*TT+1,1); %1 - pi_t, 2-y_t, 4- a_t
    x_spi = hist_sim(G1_spi,impact_spi,Sig_spi,aa(end)*TT+1,1);
    x_si = hist_sim(G1_si,impact_si,Sig_si,aa(end)*TT+1,1);
         
    % 4.2. simulated output gap
    YdatS_bT =[x_sp(2:end,1),x_spi(2:end,1),x_si(2:end,1)]; % inflation, t
    IdatS_bT =[x_sp(1:end-1,1),x_spi(1:end-1,1),x_si(1:end-1,1)]; % inflation, t-1
    YdatS_bT = YdatS_bT(:,ind);
    IdatS_bT = IdatS_bT(:,ind); 

    u = rand(aa(end)*TT,1);  

    for tsi = 1:n_aa;
        SS = aa(tsi)*TT;
        YdatS_b = YdatS_bT(1:SS,:);
        IdatS_b = IdatS_bT(1:SS,:);

        for li = 1:n_ls;
            l_val=ls(li); % l_val for actual data 

            % now, make l_vals for simulated data
            l_valS = aa(tsi)*ls(li);
            % assume that the "l_val" values are for the actual data -- for the simulated, mutiply them
            % by the factor 1, 2, or 5, as SS is made by the same factors multiplying TT 

            bdatAct = bootk1((ld_pc_Y),l_val,u);
            bdatSim = bootk1([YdatS_b IdatS_b],l_valS,u);

            % 4.3.  I carry out the usual boot approach 

            outttu = zee2o(bdatAct(:,1),bdatAct(:,2),bdatSim(:,1:size(YdatS_b,2)),...
                     bdatSim(:,size(YdatS_b,2)+1:end),Umin,Uinc,Unum);
            boboy1=mean(outttu)-Oboy(li+(tsi-1)*n_ls,:); 
            % these are the usual boot stats with the booted minus the actual 

            % 4.4. II carry out the alternative boot for S growing faster than T 

            outttu = zee2o(bdatAct(:,1),bdatAct(:,2),YdatS_b,IdatS_b,Umin,Uinc,Unum);
            boboy2=mean(outttu)-Oboy(li+(tsi-1)*n_ls,:); 
         
            % these are the alt boot stats with the booted minus the actual, ut where
            % only the actual data are resampled, and not the booted, i.e. S grow faster T 
             % calculate statistics 

           ZZb1=ZTstats(boboy1');
           ZZZb1(booti,li+(tsi-1)*n_ls)=ZZb1;

           ZZb2=ZTstats(boboy2');
           ZZZb2(booti,li+(tsi-1)*n_ls)=ZZb2;            
        end
    end
    if rem(booti,10)==0; booti; 
    end;
end  
% 4.5. construct critical values 

temp=sort(ZZZb1,1);
cv1_10=temp(floor(0.90*size(ZZZb1,1)),:);
cv1_5=temp(floor(0.95*size(ZZZb1,1)),:);

temp=sort(ZZZb2,1);
cv2_10=temp(floor(0.90*size(ZZZb2,1)),:);
cv2_5 =temp(floor(0.95*size(ZZZb2,1)),:);


% 5. Results: 
% 5.1. statistics and critical values
outza(:,2:5)=[cv1_5' cv1_10' cv2_5' cv2_10'];
% % 5.6. Actual vs simulated densities:
% 
%  j = 1;                           