function [xit, vt, ht] = KF_lik(y, mu, H, F, Q, R, xtt1, ptt1);
% =============================================
%   yt  = H*xit + et
%   xit = mu+F*xit1 + vt
%                                            
%   et = N(0, R)     R - n by n              
%   vt = N(0, Q)     Q - k by k , 
%                                 
%    xit  - k by 1
%    yt   - n by 1
%
%    INPUT:
%        H    - n by k
%        mu   - k by 1
%        P    - transition matrix
%        F    - k by k
%        R    - n by n
%        Q    - k by k
% ================================================
[T, n] = size(y);
k      = size(H,2);

xittall = zeros(k,T);               vt = zeros(n, T);
pttall  = zeros(k,T*k);             ht = zeros(n, T*n);

for j = 1:T

    fcst = y(j,:)'-H*xtt1;          % n by 1 
    ftt1 = H*ptt1*H'+R;             % n by n 
    vt(:,j) = fcst;                 % prediction error 
    ht(:, (j-1)*n+1:j*n) = ftt1;    % variance of prediction error

    K_gn = ptt1*H'*inv(ftt1);       % k by n 

    xitt = xtt1 + K_gn*fcst;        % k by 1 
    Ptt  = (eye(k)-K_gn*H)*ptt1;    % k by k 

    xtt1 = mu + F*xitt;             % k by 1 
    ptt1 = F*Ptt*F' + Q;            % k by k 

    xittall(:,j) = xitt;
    pttall(:,(j-1)*k+1:j*k) = Ptt;
end
    
T        = size(xittall,2);
xit      = xittall';

end     % end of function