function [flik vt ht] = LogLik_SP1(Y, H, Theta, Psi, Sigma_e,xtt1);
% ================================================
%    Compuation of likelihood function for sticky 
%    price model
%
%    In general, two cases are possible
%       Case 1: Likelihood is computed using KF recursions
%       Case 2: Condiditional likelihood may be computed
%               given draws of state variables
%
%    INPUT:
%       Y       - T by m matrix of observations
%       H       - m by n "selection" matrix
%       Theta   - n by n solution matrix
%       Psi     - n by m solution matrix
%       Sigma_e - m by m matrix of state error terms (may be Identity
%       matrix)
% ================================================= 
[T, m]  = size(Y);
n       = size(Theta,1);        % number of latent variables in the model 
mu      = zeros(n,1);           % 
F       = Theta;                % 
Q       = Psi*Sigma_e*Psi';     % 
R       = 0;                    % no error term in measurement equation

% =================================
%   Starting values
% =================================
%xtt1 = zeros(n,1);
Pvec = inv(eye(n*n) - kron(F,F))*Q(:);
Ptt1 = reshape(Pvec, n,n);

[Xithat, vt, ht] = KF_lik1(Y, mu, H, F, Q, R, xtt1, Ptt1);

flik = -(T/2)*log(2*pi);            % ht - variance is identity ?
for j = 1:T;
    flik = flik-.5*log(det(ht(:,(j-1)*m+1:j*m)))-.5*vt(:,j)'*inv(ht(:,(j-1)*m+1:j*m))*vt(:,j);
end