function Rho = compACF(G1, impact, Cst, Y, ai);
% function computes theoretical and empirical Auto and Cross correlation 
% functions for inflation and measure of inflationary preassure
% input: G1, impact, Cst - solution of theoretical model
%        Y               - data
%        ai              - order of ACF(ai)
% output: Rho - order (theoretical: inflation, measure, inflMeas, measInfl;
%                      empirical:  inflation, measure, inflMeas, measInfl)

Theta   = G1;
Psi     = impact;
Sig_e   = Cst*Cst';
Omega   = Psi*Sig_e*Psi';

% ============================================
%   Computation of Unconditional Variance
% ============================================
crit    = 0;                    % controls convergence
i_max   = 1000;                 % the maximum number of iterations
tol     = 2.5e-0010;             % tolerance criteria
acf_lag = 16;
V_i     = Omega;   % conditional variance
Sum_0   = V_i;                  % unconditional variance
i       = 0;
while crit == 0;
    i       = i+1;              % just a counter
    V_i   = Theta*V_i*Theta'; 
    Sum_1   = Sum_0 + V_i; 
    m_dis   = sqrt(sum((Sum_1(:)-Sum_0(:)).^2));
    if (m_dis< tol) | (i>i_max)
        crit = 1;
    else
        Sum_0 = Sum_1;
    end
end

Sig_0 = Sum_0;
% =====================================================
%   Computation of autocovariances and autocorrelations
%   X = [inflaiton ~ measure ~ income ~ interest rate
% =====================================================
rho_pi  = 1;                                        % ACF for inflation
rho_y   = 1;                                        % ACF for output
rho_pi_y= Sig_0(1,2)/sqrt(Sig_0(1,1)*Sig_0(2,2));   % cross ACF, inflation -output (?)
rho_y_pi= Sig_0(2,1)/sqrt(Sig_0(1,1)*Sig_0(2,2));   % cross ACF, output-inflation (?)

Sig_i   = Sig_0;
for i = 1:acf_lag
    Sig_i = Theta*Sig_i;                            % i-th autocovariance
    rho_pi  = [rho_pi; Sig_i(1,1)/Sig_0(1,1)];      % acf for inflaiton
    rho_y   = [rho_y; Sig_i(2,2)/Sig_0(2,2)];      % acf for inflaiton
    rho_pi_y= [rho_pi_y; Sig_i(1,2)/sqrt(Sig_0(1,1)*Sig_0(2,2))]; % cross acf
    rho_y_pi= [rho_y_pi; Sig_i(2,1)/sqrt(Sig_0(1,1)*Sig_0(2,2))]; % cross acf
end

rho_pi_dat  = acf(Y(:,1), acf_lag);     rho_pi_dat = [1; rho_pi_dat.ac];
rho_y_dat   = acf(Y(:,2), acf_lag);      rho_y_dat = [1; rho_y_dat.ac];
rho_py_dat  = cross_acf(Y(:,1), Y(:,2), acf_lag);     % inflation (lead) - output (lag) ?
rho_yp_dat  = cross_acf(Y(:,2), Y(:,1), acf_lag);     % inflation (lead) - output (lag) ?

Rho = [rho_pi(ai+1,1) rho_y(ai+1,1) rho_pi_y(ai+1,1) rho_y_pi(ai+1,1);
       rho_pi_dat(ai+1,1) rho_y_dat(ai+1,1) rho_py_dat(ai+1,1) rho_yp_dat(ai+1,1)];