function x_h = hist_sim(G1,impact,sig,T,ind);
% Function that simulates history of economy
% if solution have the following form:
% x_t =G1*x_(t-1) + impact*e_t
% e_t ~N(0,sig^2)
% T - size of simulation we need
% ____________________________________________________________________

% input arguments:
%
% G1    : output from gensys 
% impact: output from gensys  
%       : x_t =G1*x_(t-1) + impact*e_t 
% sig   : chol of variance covariance matrix of e_t, s.t. e_t ~ N(0,sig^2)
% T     : size of history simulation
% ind   : variables we want to simulate
% output:
% xh    : matrix (T,size(ind,1)) with simulated variables 
%_____________________________________________________________________
% (c) Korenok Oleg
% Date July 21, 2004

T = T + 200;
n = size(G1,1);
x_h = zeros(n,T);
e = (sig*randn(size(sig,1),T))';
x_h(:,1) = impact*e(1,:)';

for i = 2:T;
    x_h(:,i) = G1*x_h(:,i-1) + impact*e(i,:)';
end;
x_h = x_h';
x_h = x_h(201:end,ind);