% ==================================================================
% This program computes and plots likelihood functions for different 
% for all three models 
% Korenok, Radchenko, Swanson "Stickiness: International Comparison"
% February 6, 2006
% ==================================================================
%clc

clear;
% step 1. Load the data
% example US data; load raw data
Icomp = 1;
if Icomp == 1;
    dataplace = 'C:\Documents and Settings\Faculty\Desktop\current projects\infoIV\data';
    progplace = 'C:\Documents and Settings\Faculty\Desktop\current projects\infoIV\mlab';
else
    dataplace = 'H:\My Documents\SPSImodels\data';
    progplace = 'H:\My Documents\SPSImodels\matlab';
end

% cd(dataplace);
% filename = 'OECDEconomicOutlook77.xls';     range   = 'AE7:AE188'; % AE - U.S.
% rgdp    = xlsread(filename, 2, range);
% yg      = xlsread(filename, 5, range);
% pop     = xlsread(filename, 8, range);
% gdpDef  = xlsread(filename, 9, range);
% ulc      = xlsread(filename, 3, range); 
% unemp   = xlsread(filename, 6, range);
% intr    = xlsread(filename, 12, range);
% cd(progplace);
% 
% % ======================================
% %   Variables transformations
% % ======================================
% dates = (1960.25:.25:2005.5)';
% % this is just for output gap, excel related
% yg(yg==-10000,:)    = 0;
% yg(yg<-10,:)        = yg(yg<-10,:)+100000;
% yg                  = yg/100;
% 
% rulc = log(ulc) - log(gdpDef); % real unit labor cost (labor share) 
% dulc = log(ulc(2:end)) - log(ulc(1:end-1)); % quarterly inflation in ulc
% unemp = unemp/100;
% intrQuart              = (1+intr/100).^(1/4)-1;     % quarterly interest rate
% [piDefQuart piDefYear] = Quart_trans(gdpDef);       % quaterly inflation
% [rgdpPerCapitaGrQuart rgdpPerCapitaGrYear] = Quart_trans(rgdp./pop); % quarterly per capita output
% % plot 3 different measures
% %figure; plot(dates,[yg -unemp rulc]); label('y^g','u_t', 'rulc_t');
% %corrcoef([yg(96:end) -unemp(96:end) rulc(96:end)]);
 
indCount = [1,4,10,11,12,15,17,18,22,23,24,28,30]; % selecting countries
%indCount = [1,4,10,11,12,15,18,22,23,24,28,30]; % without Italy
%Aust, Can, Fin, Fra, UK, Ire, Ita, Jap, Neth, Nor, NZ, Swe, US
cd(dataplace);
filename = 'OECDEconomicOutlook77.xls';     range   = 'B7:AE188'; 
rgdp    = xlsread(filename, 2, range); rgdp = rgdp(:,indCount);
yg      = xlsread(filename, 5, range); yg = yg(:,indCount);
pop     = xlsread(filename, 8, range); pop = pop(:,indCount);
gdpDef  = xlsread(filename, 9, range); gdpDef = gdpDef(:,indCount);
ulc     = xlsread(filename, 3, range); ulc = ulc(:,indCount);
unemp   = xlsread(filename, 6, range); unemp = unemp(:,indCount);
intr    = xlsread(filename, 12, range);intr = intr(:,indCount);
cd(progplace);

par_sp = xlsread('res_mle2', 1, 'C3:K54');
par_spi = xlsread('res_mle2', 2, 'C3:K54');
par_si  = xlsread('res_mle2', 3, 'C3:N54');

% ======================================
%   Variables transformations
% ======================================
dates = (1960.25:.25:2005.5)';
[tC, nC] = size(yg);
rulc= zeros(tC,nC); unempA = rulc;
intrQuart = rulc; intrA = rulc; 
piDefQuart =zeros(tC-1,nC); piDefYear = zeros(tC-4,nC);
for i=1:nC
    yg(yg(:,i)==-10000,i)    = 0;                   % this is just for output gap, excel related
    yg(yg(:,i)<-10,i)        = yg(yg(:,i)<-10,i)+100000;
    yg(:,i)                  = yg(:,i)/100;
    ind  = find(ulc(:,i)~=-10000);
    rulc(ind,i) = log(ulc(ind,i)) - log(gdpDef(ind,i)); % real unit labor cost (labor share) 
    %dulc(ind,i) = log(ulc(ind(2:end),i)) - log(ulc(ind(1:end-1),i)); % quarterly inflation in ulc
    ind  = find(unemp(:,i)~=-10000);
    unempA(ind,i) = unemp(ind,i)/100;
    unemp(:,i) = unempA(:,i);
    ind  = find(intr(:,i)~=-10000);
    intrQuart(ind,i)= (1+intr(ind,i)/100).^(1/4)-1;     % quarterly interest rate
    intrA(ind,i) = intr(ind,i)/100;
    intr(:,i) = intrA(:,i);
    %figure; plot(dates,intr(:,i));
    ind  = find(gdpDef(:,i)~=-10000);
    [piDefQuart(ind(1:end-1),i) piDefYear(ind(1:end-4),i)] = Quart_trans(gdpDef(ind,i));       % quaterly inflation
    ind  = find(rgdp(:,i)~=-10000);
    [rgdpPerCapitaGrQuart(ind(1:end-1),i) rgdpPerCapitaGrYear(ind(1:end-4),i)]...
        = Quart_trans((rgdp(ind,i)./pop(ind,i))); % quarterly per capita output
    %figure; plot(dates(5:end),rgdpPerCapitaGrYear(:,i));
end


% ===========================================
%   X data for VAR
% ===========================================
nvar = 4;
% Uncomment for alternatives to output gap:
% yg = rulc; % using rulc instead of output gap
%X = [ piDefQuart(1:end)  yg(2:end) rgdpPerCapitaGrQuart(1:end)   intrQuart(2:end)];
% yg = -unemp;% using unemployment instead of output gap
%X = [ piDefQuart(17:end)  yg(18:end) rgdpPerCapitaGrQuart(17:end)   intrQuart(18:end)];
%X = [ piDefYear(14:end) yg(18:end) rgdpPerCapitaGrYear(14:end)   intr(18:end)/100];
%X = X(75:end,:); % only when using rulc 

I_mes       =1;        % output gap measure: 
                        % 1  - "output" gap; 2 - unit labor cost; 3- unemployment
I_mod       = 1;        % 1- sp; 2 - spi;   3 - si                        
I_freq      = 1;        % 1- quarterly defenition; 2 - annual defenition
I_subsample = 1;        % 1 - full sample
I_country   = 13;

 X  = def_X(piDefQuart,  yg, rulc, unemp, rgdpPerCapitaGrQuart, intrQuart,...
 piDefYear, rgdpPerCapitaGrYear, intr,dates, I_freq, I_subsample, I_mes, I_country);

X = X - ones(size(X,1),1)*mean(X);
nlag = 1; 

Xlag = [];          
for j=1:nlag
    Xlag = [Xlag X(nlag-j+1:end-j,:)];
end
Y = X(nlag+1:end,:);
X = Xlag;

% =============================================================
% step 2.1. Estimate parameters of reduced equations (VAR)
%           B = X\Y is the solution to the equation XB = Y
% =============================================================
B   = X\Y;
err = Y - X*B;
C   = chol(cov(err))';
% Y[t] = X[t]*B + w[t]*C
% Calculating cholesky decomposition for structural model
Cs      = chol(cov(err(:,2:end)))'; % covariance of tw[t]
se_v    = C(1,1); % covariance of v[t]
Cst     = eye(nvar); Cst(1,1) = se_v; Cst(2:end,2:end) = Cs;
% matrix in companion form
B = [B'; eye(nvar*(nlag-1)) zeros(nvar*(nlag-1),nvar)];

% =============================================================
% step 2.2. Specify Priors and Parameters Starting values
% =============================================================
b   = .99;  % beta, consumer discount factor, only for sp or spi models

%         % this is based on "standard" calibration
%         a   = 2/3;  % labor share
%         psi = 1;    % psi, intertemporal subst of labor 
%         s   = 1;    % intertemporal substitution of consumption
%         eps = 11;   % Dixit-Stiglitz parameter
%         th  = .75;   % price stickiness
%         om = psi/a + 1/a -1;         % omega
%         xi = (s+om)/(1+om*eps);      % xi
%         la = (1-th)*(1-b*th)/th*xi;     % lambda

la  = 0.0112;   % lambda, only for sticky price or sticky price 
                % with indexation (nonlinear combination of stickiness, 
                % and other structural parameters)
th  = 0.9;     % information stickiness parameter (only for sticky 
                % information model) 
xi  = 0.1304;   % nonlinear combination of deep structural parameters
                % only for sticky information model

spar = [b;la;th; xi]; % set of structural parameters, la - for sp, spi models
                      % th and xi for si model
                      % b - for parameter for all three models
                      
rpar = B(2:end,:); % set of reduced form parameters

% spar(2)=-.05;
% [G1, impact, eu] = solveTeoreticalModel(spar, rpar, nvar, 2);
% H = [eye(4) zeros(nvar,size(G1,1)-nvar)]; %"selection" matrix
% xtt1 = zeros(size(G1,1),1); xtt1(1:size(X(1,:),2)) = (X(1,:)*B')';
% [flik vt ht] = LogLik_SP1(Y, H , G1, impact, Cst*Cst',xtt1);
% i=1;

% % =============================================================
% % step 3.1. Likelihood Function for Sticky price model
% % =============================================================
% % Sticky price model has 3 structural parameters that we estimate
% % beta, lambda and variance of structural shock, vt
% 
% % negative in the case of ulc measure, positive for the output gap and 
% % unemployment
% %lam = (0.01:0.1:1)'; % range for lambda, case 1
% lam = (-0.01:0.001:.01)'; % range for lambda, case 2
% 
% % 3.1.1. Only for lambda                
% nrep = size(lam,1);
% flikm = zeros(nrep,1);
% for j =1:nrep;
%     spar(2) = lam(j);    
%     [G1, impact, eu] = solveTeoreticalModel(spar, rpar, nvar, 1);
%     if sum(eu)~=2; flikm(j,:) = 0; continue; end;
%     H = [eye(4) zeros(nvar,size(G1,1)-nvar)]; %"selection" matrix
%     xtt1 = zeros(size(G1,1),1); xtt1(1:size(X(1,:),2)) = (X(1,:)*B')';
%     [flik vt ht] = LogLik_SP1(Y, H , G1, impact, Cst*Cst',xtt1);
%     flikm(j,:) = flik; 
% end
% flikm(flikm==0,:) = min(min(flikm));
% figure; plot(lam,flikm); title('likelihood function for parameter \lambda')


% % conclusion: strictly decreasing in lambda for 
% 
% % 3.1.2. For lambda and v
% lam = (-0.0066:0.002:.0066)'; % range for lambda, case 2
% vm  = (0.004:.001:.008)'; % range for vt
% nrepl = size(lam,1);
% nrepv = size(vm,1);
% flikm = zeros(nrepl,nrepv);
% for j =1:nrepl;
%     spar(2) = lam(j);
%     for i = 1:nrepv;
%         Cst(1,1) = vm(i);
%         [G1, impact, eu] = solveTeoreticalModel(spar, rpar, nvar, 1);
%         if sum(eu)~=2; flikm(j,i) = 0; continue; end;
%         H = [eye(nvar) zeros(nvar,size(G1,1)-nvar)]; %"selection" matrix
%         xtt1 = zeros(size(G1,1),1); xtt1(1:size(X(1,:),2)) = (X(1,:)*B')';
%         [flik vt ht] = LogLik_SP1(Y, H , G1, impact, Cst*Cst',xtt1);
%         flikm(j,i) = flik;
%     end
% end
% %flikm(flikm==0,:) = min(min(flikm));
% figure; surf(vm,lam,flikm); xlabel('\sigma_v'); ylabel('\lambda_1'); zlabel('Log Likelihood');
% ind = 1;
% conclusion: strictly decreasing in lambda for all values of v

% % 3.1.3. For lambda and beta
% bm  = (0.97:.005:.995)'; % range for vt
% nrepl = size(lam,1);
% nrepb = size(bm,1);
% flikm = zeros(nrepl,nrepb);
% for j =1:nrepl;
%     spar(2) = lam(j);
%     for i = 1:nrepb;
%         spar(1) = bm(i);
%         [G1, impact, eu] = solveTeoreticalModel(spar, rpar, nvar, 1);
%         if sum(eu)~=2; flikm(j,i) = 0; continue; end;
%         H = [eye(nvar) zeros(nvar,size(G1,1)-nvar)]; %"selection" matrix
%         xtt1 = zeros(size(G1,1),1); xtt1(1:size(X(1,:),2)) = (X(1,:)*B')';
%         [flik vt ht] = LogLik_SP1(Y, H , G1, impact, Cst*Cst',xtt1);
%         flikm(j,i) = flik;
%     end
% end
% %flikm(flikm==0,:) = min(min(flikm));
% figure; surf(bm,lam,flikm); xlabel('\beta'); ylabel('\lambda'); zlabel('loglik');
% % conclusion: strictly decreasing in lambda for all values of beta

% Main conclusion mle estimate of lambda goes to zero, this is close to
% Kiley result, he finds very small estimate of lambda, though he uses different
% data set

% =============================================================
% step 3.2. Likelihood Function for Sticky price model with indexation
% =============================================================
% Sticky price model with indexation has 3 structural parameters that we estimate
% beta, lambda and variance of structural shock, vt

% negative in the case of ulc measure, positive for the output gap and 
% unemployment
%lam = -(0.001:0.001:.1)'; % range for lambda, case 2

% % 3.1.1. Only for lambda                
% nrep = size(lam,1);
% flikm = zeros(nrep,1);
% for j =1:nrep;
%     spar(2) = lam(j);    
%     [G1, impact, eu] = solveTeoreticalModel(spar, rpar, nvar, 2);
%     if sum(eu)~=2; flikm(j,:) = 0; continue; end;
%     H = [eye(4) zeros(nvar,size(G1,1)-nvar)]; %"selection" matrix
%     xtt1 = zeros(size(G1,1),1); xtt1(1:size(X(1,:),2)) = (X(1,:)*B')';
%     [flik vt ht] = LogLik_SP1(Y, H , G1, impact, Cst*Cst',xtt1);
%     flikm(j,:) = flik; 
% end
% %flikm(flikm==0,:) = min(min(flikm));
% figure; plot(lam,flikm); title('likelihood function for parameter \lambda')
% % conclusion: likelihood is flat for lam = (0.001:0.01:.1)'  

% 3.1.2. For lambda and v

lam = (0.001:0.001:.01)'; % range for lambda, case 2
vm  = (0.001:.0001:.004)'; % range for vt
nrepl = size(lam,1);
nrepv = size(vm,1);
flikm = zeros(nrepl,nrepv);
for j =1:nrepl;
    spar(2) = lam(j);
    for i = 1:nrepv;
        Cst(1,1) = vm(i);
        [G1, impact, eu] = solveTeoreticalModel(spar, rpar, nvar, 2);
        if sum(eu)~=2; flikm(j,i) = 0; continue; end;
        H = [eye(nvar) zeros(nvar,size(G1,1)-nvar)]; %"selection" matrix
        xtt1 = zeros(size(G1,1),1); xtt1(1:size(X(1,:),2)) = (X(1,:)*B')';
        [flik vt ht] = LogLik_SP1(Y, H , G1, impact, Cst*Cst',xtt1);
        flikm(j,i) = flik;
    end
end
%flikm(flikm==0,:) = min(min(flikm));
figure; surf(vm,lam,flikm); xlabel('\sigma_v'); ylabel('\lambda_2'); zlabel('Log Likelihood');
ind = 1;
%conclusion: likelihood is flat for lam = (0.001:0.01:.1)' all v

% % 3.1.3. For lambda and beta
% bm  = (0.97:.005:.995)'; % range for vt
% nrepl = size(lam,1);
% nrepb = size(bm,1);
% flikm = zeros(nrepl,nrepb);
% for j =1:nrepl;
%     spar(2) = lam(j);
%     for i = 1:nrepb;
%         spar(1) = bm(i);
%         [G1, impact, eu] = solveTeoreticalModel(spar, rpar, nvar, 2);
%         if sum(eu)~=2; flikm(j,i) = 0; continue; end;
%         H = [eye(nvar) zeros(nvar,size(G1,1)-nvar)]; %"selection" matrix
%         xtt1 = zeros(size(G1,1),1); xtt1(1:size(X(1,:),2)) = (X(1,:)*B')';
%         [flik vt ht] = LogLik_SP1(Y, H , G1, impact, Cst*Cst',xtt1);
%         flikm(j,i) = flik;
%     end
% end
% %flikm(flikm==0,:) = min(min(flikm));
% figure; surf(bm,lam,flikm); xlabel('\beta'); ylabel('\lambda'); zlabel('loglik');
% % conclusion: likelihood is flat for lam = (0.001:0.01:.1)' for all beta
% 
% % general conclusion: for the output gap likelihood is flat, for
% % unemployment - max at small lambda, for rulc - small negative lambda
% 

% % =============================================================
% % step 3.3. Likelihood Function for Sticky Information Models
% % =============================================================
% % Sticky information model with indexation has 3 structural parameters that we estimate
% % theta, xi and variance of structural shock, vt
% % 
% 
% % 3.1.1. Only for lambda                
% nrep = size(thm,1);
% flikm = zeros(nrep,1);
% for j =1:nrep;
%     spar(3) = thm(j);    
%     [G1, impact, eu] = solveTeoreticalModel(spar, rpar, nvar, 3);
%     if sum(eu)~=2; flikm(j,:) = 0; continue; end;
%     H = [eye(nvar) zeros(nvar,size(G1,1)-nvar)]; %"selection" matrix
%     xtt1 = zeros(size(G1,1),1); xtt1(1:size(X(1,:),2)) = (X(1,:)*B')';
%     [flik vt ht] = LogLik_SP1(Y, H , G1, impact, Cst*Cst',xtt1);
%     flikm(j,:) = flik; 
% end
% %flikm(flikm==0,:) = min(min(flikm));
% figure; plot(thm,flikm); title('likelihood function for parameter \theta')
% 
% % conclusion: likelihood has max when theta close to 1
% % likelihood has max around theta = .65 (rulc) 
% spar(4) = -.0233;
% thm = (0.6:0.05:1)'; % range for theta
% % 3.1.2. For lambda and v
% vm  = (0.003:.001:.008)'; % range for vt
% nrepl = size(thm,1);
% nrepv = size(vm,1);
% flikm = zeros(nrepl,nrepv);
% for j =1:nrepl;
%     spar(3) = thm(j);
%     for i = 1:nrepv;
%         Cst(1,1) = vm(i);
%         [G1, impact, eu] = solveTeoreticalModel(spar, rpar, nvar, 3);
%         if sum(eu)~=2; flikm(j,i) = 0; continue; end;
%         H = [eye(nvar) zeros(nvar,size(G1,1)-nvar)]; %"selection" matrix
%         xtt1 = zeros(size(G1,1),1); xtt1(1:size(X(1,:),2)) = (X(1,:)*B')';
%         [flik vt ht] = LogLik_SP1(Y, H , G1, impact, Cst*Cst',xtt1);
%         flikm(j,i) = flik;
%     end
% end
% %flikm(flikm==0,:) = min(min(flikm));
% figure; surf(vm,thm,flikm); xlabel('\sigma_v'); ylabel('\theta_3'); zlabel('Log Likelihood');
% ind = 1;
% % conclusion: likelihood has max when theta close to 1, independent of v 
% % as v decreases , likelihood max at lower theta, (rulc) 

% 3.1.3. For lambda and beta
Cst(1,1) = 0.0059; 
thm = (0.6:0.05:1)'; % range for theta
xim  = -(0.0001:.01:.04)'; % range for vt
nrepl = size(thm,1);
nrepb = size(xim,1);
flikm = zeros(nrepl,nrepb);
for j =1:nrepl;
    spar(3) = thm(j);
    for i = 1:nrepb;
        spar(4) = xim(i);
        [G1, impact, eu] = solveTeoreticalModel(spar, rpar, nvar, 3);
        if sum(eu)~=2; flikm(j,i) = 0; continue; end;
        H = [eye(nvar) zeros(nvar,size(G1,1)-nvar)]; %"selection" matrix
        xtt1 = zeros(size(G1,1),1); xtt1(1:size(X(1,:),2)) = (X(1,:)*B')';
        [flik vt ht] = LogLik_SP1(Y, H , G1, impact, Cst*Cst',xtt1);
        flikm(j,i) = flik;
    end
end
%flikm(flikm==0,:) = min(min(flikm));
figure; surf(xim,thm,flikm); xlabel('\xi'); ylabel('\theta_3'); zlabel('Log Likelihood');
 ind = 1;

% 

% general conclusion: 
